@@ -44,6 +44,10 @@ def decodeCustomTx(hex,DfMarker):
             res = decodeCreatePoolPair(hex[2:])
             if res != None:
                 res['name'] = 'createpoolpair'
+        elif functionMarker == 'u':
+            res = decodeUpdatePoolPair(hex[2:])
+            if res != None:
+                res['name'] = 'updatepoolpair'     
         elif functionMarker == 'B':
             res = decodeAccountToAccount(hex[2:])
             if res != None:
@@ -155,6 +159,16 @@ def decodeSetGovVariable(hex,withHeight):
                 ]
                 result['attributes'].append(searchHex(variables,hex[8:50]))
                 hex = hex[50:]
+            elif typeMarker in ['t']: 
+                variables = [
+                    {'bytesCount': 1, 'key': 'type', 'type': str,'endian':None},
+                    {'bytesCount': 4, 'key': 'tokenId', 'type': int,'endian':'little'},
+                    {'bytesCount': 1, 'key': 'key', 'type': str,'endian':None},
+                    {'bytesCount': 4, 'key': 'keyId', 'type': int,'endian':'little'},
+                    {'bytesCount': 8, 'key': 'amount', 'type': int,'endian':'little'}
+                ]
+                result['attributes'].append(searchHex(variables,hex[8:44]))
+                hex = hex[44:]
 
         if withHeight: 
             result['height'] = int(utils.convert_hex(hex[-8:],'little','big'),16)
@@ -179,6 +193,8 @@ def decodeSetGovVariable(hex,withHeight):
             'hex': hex
         }
     elif utils.stringToHex('LP_LOAN_TOKEN_SPLITS') == govVariable:
+        # Anzahl der Splits
+        # Token ID und % Anteil an den Rewards
         return {
             'govvariable': 'LP_LOAN_TOKEN_SPLITS',
             'hex': hex
@@ -199,10 +215,10 @@ def decodeSetGovVariable(hex,withHeight):
 def decodePoolSwap(hex):
     variables = [
         {'key': 'from', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'fromAmount', 'type': int,'endian':'little'},
         {'key': 'to', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'maxPriceCounter', 'type': int,'endian':'little'},
         {'bytesCount': 8, 'key': 'maxPriceDenominator', 'type': int,'endian':'little'}
     ]
@@ -210,23 +226,23 @@ def decodePoolSwap(hex):
 def decodePoolSwapV2(hex):
     variables = [
         {'key': 'from', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'fromAmount', 'type': int,'endian':'little'},
         {'key': 'to', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'maxPriceCounter', 'type': int,'endian':'little'},
         {'bytesCount': 8, 'key': 'maxPriceDenominator', 'type': int,'endian':'little'},
         {'key':'pools', 'type':
-            [{'bytesCount': 1, 'type': int,'endian':None}]
+            [{'bytesCount': 1, 'type': int,'endian':None, 'canExceedBytesCount': True}]
         }
     ]
     return searchHex(variables,hex)
 def decodeDFIP2203(hex):
     variables = [
         {'key': 'from', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'fromTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'fromAmount', 'type': int,'endian':'little'},
-        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'toTokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount':4, 'key': 'direction', 'type': int,'endian':None}
     ]
     return searchHex(variables,hex)
@@ -246,20 +262,28 @@ def decodeAddPoolLiquidity(hex):
 def decodeRemovePoolLiquidity(hex):
     variables = [
         {'key': 'from', 'type': 'address','endian':None},
-        {'bytesCount':1, 'key': 'tokenId', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'tokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'amount', 'type': int,'endian':'little'}
     ]
     return searchHex(variables,hex)
 def decodeCreatePoolPair(hex):
     variables = [
-        {'bytesCount':1, 'key': 'tokenIdA', 'type': int,'endian':None},
-        {'bytesCount':1, 'key': 'tokenIdB', 'type': int,'endian':None},
+        {'bytesCount':1, 'key': 'tokenIdA', 'type': int,'endian':None, 'canExceedBytesCount': True},
+        {'bytesCount':1, 'key': 'tokenIdB', 'type': int,'endian':None, 'canExceedBytesCount': True},
         {'bytesCount': 8, 'key': 'commission', 'type': int,'endian':'little'},
         {'key': 'from', 'type': 'address','endian':None},
         {'bytesCount':1, 'key': 'status', 'type': int,'endian':None},
         {'key': 'customRewards', 'type': 'rest' ,'endian':None}
     ]
     return searchHex(variables,hex)
+def decodeUpdatePoolPair(hex):
+    variables = [
+        {'bytesCount':1, 'key': 'tokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
+        {'bytesCount':4, 'key': 'status', 'type': int,'endian':None},
+        {'bytesCount': 8, 'key': 'commission', 'type': int,'endian':'little'},
+        {'bytesCount': 'rest', 'key': 'rest', 'type': str,'endian':None}
+    ]
+    return searchHex(variables,hex)
 def decodeAccountToAccount(hex):
     variables = [
         {'key': 'from', 'type': 'address','endian':None},
@@ -297,7 +321,7 @@ def decodePaybackLoanV2(hex):
         {'bytesCount': 32,'key': 'vaultId', 'type': 'hex','endian':'little', 'keepLeadingZeros': True},
         {'key': 'from', 'type': 'address','endian':None},
         {'key': 'tokens', 'type': [
-            {'bytesCount': 1,'key': 'tokenId', 'type': int,'endian':None},
+            {'bytesCount': 1,'key': 'tokenId', 'type': int,'endian':None, 'canExceedBytesCount': True},
             {'key': 'amounts', 'type': [
                 {'bytesCount': 4,'key': 'tokenId', 'type': int,'endian':'little'},
                 {'bytesCount': 8, 'key': 'amount', 'type': int,'endian':'little'}
@@ -349,7 +373,7 @@ def decodeAuctionBid(hex):
         {'bytesCount': 32, 'key': 'vaultId', 'type': 'hex', 'endian':'little', 'keepLeadingZeros': True},
         {'bytesCount': 4,'key': 'index', 'type': int,'endian':'little'},
         {'key': 'address', 'type': 'address','endian':None},
-        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little'},
+        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little', 'canExceedBytesCount': True},
         {'bytesCount': 8,'key': 'amount', 'type': int,'endian':'little'}
     ]
     return searchHex(variables,hex)
@@ -357,7 +381,7 @@ def decodeWithdrawFromVault(hex):
     variables = [
         {'bytesCount': 32, 'key': 'vaultId', 'type': 'hex', 'endian':'little', 'keepLeadingZeros': True},
         {'key': 'to', 'type': 'address','endian':None},
-        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little'},
+        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little', 'canExceedBytesCount': True},
         {'bytesCount': 8,'key': 'amount', 'type': int,'endian':'little'}
     ]
     return searchHex(variables,hex)
@@ -365,7 +389,7 @@ def decodeDepositToVault(hex):
     variables = [
         {'bytesCount': 32, 'key': 'vaultId', 'type': 'hex', 'endian':'little', 'keepLeadingZeros': True},
         {'key': 'from', 'type': 'address','endian':None},
-        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little'},
+        {'bytesCount': 1, 'key': 'tokenId', 'type': int,'endian':'little', 'canExceedBytesCount': True},
         {'bytesCount': 8,'key': 'amount', 'type': int,'endian':'little'}
     ]
     return searchHex(variables,hex)
@@ -459,8 +483,21 @@ def getValue(variable,hex):
         elif variable['bytesCount'] == 0:
             hexOfInterest = ''
         else:
-            hexOfInterest = hex[:2*variable['bytesCount']]
-            hex = hex[2*variable['bytesCount']:]
+            if variable['type'] == int and 'canExceedBytesCount' in variable and variable['canExceedBytesCount']:
+                hexTemp = hex
+                counter = 0
+                while True:
+                    counter += 1
+                    hexCheck = hexTemp[:2*variable['bytesCount']]
+                    if int(hexCheck,16) == 128:
+                        hexTemp = hexTemp[2*variable['bytesCount']:]
+                    else:
+                        hexOfInterest = hex[:counter*2*variable['bytesCount']]
+                        hex = hex[counter*2*variable['bytesCount']:]   
+                        break
+            else:
+                hexOfInterest = hex[:2*variable['bytesCount']]
+                hex = hex[2*variable['bytesCount']:]
 
     # Handle Endian Transformation
     hexZeros = ''
@@ -483,7 +520,12 @@ def getValue(variable,hex):
         if hexOfInterest == '':
             valueOfInterest = 0
         else:
-            valueOfInterest = int(hexOfInterest,16)
+            if 'canExceedBytesCount' in variable and variable['canExceedBytesCount']:
+                valueOfInterest = 0
+                for i in range(0,int(len(hexOfInterest)/2/variable['bytesCount'])):
+                    valueOfInterest += int(hexOfInterest[i*2*variable['bytesCount']:(i+1)*2*variable['bytesCount']],16)
+            else:
+                valueOfInterest = int(hexOfInterest,16)
     elif variable['type'] == str:
         if hexOfInterest == '':
             valueOfInterest = hexOfInterest
